@@ -53,7 +53,7 @@ display_add_error (GdmDisplay *d)
 {
 	if (errno != 0)
 		gdm_error (_("%s: Could not write new authorization entry: %s"),
-			   "add_auth_entry", g_strerror (errno));
+			   "add_auth_entry", strerror (errno));
 	else
 		gdm_error (_("%s: Could not write new authorization entry.  "
 			     "Possibly out of diskspace"),
@@ -63,7 +63,7 @@ display_add_error (GdmDisplay *d)
 			(_("GDM could not write a new authorization "
 			   "entry to disk.  Possibly out of diskspace.%s%s"),
 			 errno != 0 ? "  Error: " : "",
-			 errno != 0 ? g_strerror (errno) : "");
+			 errno != 0 ? strerror (errno) : "");
 		gdm_text_message_dialog (s);
 		g_free (s);
 	}
@@ -424,7 +424,7 @@ try_user_add_again:
     if (automatic_tmp_dir ||
 	authdir == NULL ||
 	! gdm_file_check ("gdm_auth_user_add", user, authdir, GdmUserAuthFile, 
-			  TRUE, GdmUserMaxFile, GdmRelaxPerms) ||
+			  TRUE, FALSE, GdmUserMaxFile, GdmRelaxPerms) ||
 	! try_open_append (d->userauth)) {
 
 	/* No go. Let's create a fallback file in GdmUserAuthFB (/tmp) */
@@ -434,7 +434,9 @@ try_user_add_again:
 	authfd = g_mkstemp (d->userauth);
 
 	if (authfd == -1) {
-	    gdm_error (_("gdm_auth_user_add: Could not open cookie file %s"), d->userauth);
+	    gdm_error (_("%s: Could not open cookie file %s"),
+		       "gdm_auth_user_add",
+		       d->userauth);
 	    g_free (d->userauth);
 	    d->userauth = NULL;
 
@@ -451,7 +453,9 @@ try_user_add_again:
 
 	/* FIXME: Better implement my own locking. The libXau one is not kosher */
 	if (XauLockAuth (d->userauth, 3, 3, 0) != LOCK_SUCCESS) {
-	    gdm_error (_("gdm_auth_user_add: Could not lock cookie file %s"), d->userauth);
+	    gdm_error (_("%s: Could not lock cookie file %s"),
+		       "gdm_auth_user_add",
+		       d->userauth);
 	    g_free (d->userauth);
 	    d->userauth = NULL;
 
@@ -468,7 +472,9 @@ try_user_add_again:
 
     if (!af) {
 	/* Really no need to clean up here - this process is a goner anyway */
-	gdm_error (_("gdm_auth_user_add: Could not open cookie file %s"), d->userauth);
+	gdm_error (_("%s: Could not open cookie file %s"),
+		   "gdm_auth_user_add",
+		   d->userauth);
 	if (locked)
 		XauUnlockAuth (d->userauth);
 	g_free (d->userauth);
@@ -568,10 +574,12 @@ gdm_auth_user_remove (GdmDisplay *d, uid_t user)
      * to it. So we better play it safe... */
 
     if ( ! gdm_file_check ("gdm_auth_user_remove", user, authdir, authfile, 
-			   TRUE, GdmUserMaxFile, GdmRelaxPerms)) {
+			   TRUE, FALSE, GdmUserMaxFile, GdmRelaxPerms)) {
 	    g_free (authdir);
 	    g_free (authfile);
-	    gdm_error (_("gdm_auth_user_remove: Ignoring suspiciously looking cookie file %s"), d->userauth);
+	    gdm_error (_("%s: Ignoring suspiciously looking cookie file %s"),
+		       "gdm_auth_user_remove",
+		       d->userauth);
 
 	    return; 
     }
@@ -670,11 +678,5 @@ gdm_auth_purge (GdmDisplay *d, FILE *af)
     g_slist_free (keep);
 }
 
-void
-gdm_auth_set_local_auth (GdmDisplay *d)
-{
-	XSetAuthorization ((char *)"MIT-MAGIC-COOKIE-1", (int) strlen ("MIT-MAGIC-COOKIE-1"),
-			   (char *)d->bcookie, (int) 16);
-}
 
 /* EOF */
