@@ -26,12 +26,7 @@ void		gdm_error_box_full	(GdmDisplay *d,
 					 GtkMessageType type,
 					 const char *error,
 					 const char *details_label,
-					 const char *details_file,
-					 /* zero doesn't mean root,
-					    we never wish to run as root,
-					    zero means use the gdm user */
-					 uid_t uid,
-					 gid_t gid);
+					 const char *details_file);
 
 void		gdm_error_box		(GdmDisplay *d,
 					 GtkMessageType type,
@@ -43,6 +38,9 @@ char *		gdm_failsafe_question	(GdmDisplay *d,
 
 gboolean	gdm_failsafe_yesno	(GdmDisplay *d,
 					 const char *question);
+int		gdm_failsafe_ask_buttons (GdmDisplay *d,
+					  const char *question,
+					  char **but);
 
 #endif /* GDM_ERRORGUI_H */
 
