@@ -50,6 +50,7 @@
 #include "verify.h"
 #include "display.h"
 #include "choose.h"
+#include "getvt.h"
 #include "gdm-net.h"
 
 /* Local functions */
@@ -84,6 +85,8 @@ GdmConnection *unixconn = NULL; /* UNIX Socket connection */
 
 char *gdm_charset = NULL;
 
+int gdm_normal_runlevel = -1; /* runlevel on linux that gdm was started in */
+
 /* True if the server that was run was in actuallity not specified in the
  * config file.  That is if xdmcp was disabled and no local servers were
  * defined.  If the user kills all his local servers by mistake and keeps
@@ -100,24 +103,32 @@ int gdm_in_signal = 0;
 gchar *GdmUser = NULL;
 gchar *GdmGroup = NULL;
 gchar *GdmSessDir = NULL;
+gchar *GdmXsession = NULL;
 gchar *GdmLocaleFile = NULL;
+#if 0
+/* FIXME: Maybe just whack this */
 gchar *GdmGnomeDefaultSession = NULL;
+#endif
 gchar *GdmAutomaticLogin = NULL;
 gboolean GdmAutomaticLoginEnable = FALSE;
-gchar *GdmLocalNoPasswordUsers = NULL;
 gboolean GdmAlwaysRestartServer = FALSE;
 gchar *GdmConfigurator = NULL;
 gboolean GdmConfigAvailable = FALSE;
 gboolean GdmSystemMenu = FALSE;
+gboolean GdmChooserButton = FALSE;
 gboolean GdmBrowser = FALSE;
+gboolean GdmAddGtkModules = FALSE;
+gboolean GdmDoubleLoginWarning = TRUE;
 gchar *GdmGlobalFaceDir = NULL;
 gint GdmXineramaScreen = 0;
 gchar *GdmGreeter = NULL;
 gchar *GdmRemoteGreeter = NULL;
+gchar *GdmGtkModulesList = NULL;
 gchar *GdmChooser = NULL;
 gchar *GdmLogDir = NULL;
 gchar *GdmDisplayInit = NULL;
 gchar *GdmSessionDir = NULL;
+gchar *GdmPostLogin = NULL;
 gchar *GdmPreSession = NULL;
 gchar *GdmPostSession = NULL;
 gchar *GdmFailsafeXServer = NULL;
@@ -163,6 +174,7 @@ gint  GdmFlexibleXServers = 5;
 gchar *GdmXnest = NULL;
 int GdmFirstVT = 7;
 gboolean GdmVTAllocation = TRUE;
+gboolean GdmDisallowTCP = TRUE;
 
 
 /* set in the main function */
@@ -236,16 +248,19 @@ gdm_config_parse (void)
     GdmDisplayInit = gnome_config_get_string (GDM_KEY_INITDIR);
     GdmAutomaticLoginEnable = gnome_config_get_bool (GDM_KEY_AUTOMATICLOGIN_ENABLE);
     GdmAutomaticLogin = gnome_config_get_string (GDM_KEY_AUTOMATICLOGIN);
-    GdmLocalNoPasswordUsers = gnome_config_get_string (GDM_KEY_LOCALNOPASSWORDUSERS);
     GdmAlwaysRestartServer = gnome_config_get_bool (GDM_KEY_ALWAYSRESTARTSERVER);
     GdmGreeter = gnome_config_get_string (GDM_KEY_GREETER);
     GdmRemoteGreeter = gnome_config_get_string (GDM_KEY_REMOTEGREETER);
+    GdmAddGtkModules = gnome_config_get_bool (GDM_KEY_ADD_GTK_MODULES);
+    GdmDoubleLoginWarning = gnome_config_get_bool (GDM_KEY_DOUBLELOGINWARNING);
+    GdmGtkModulesList = gnome_config_get_string (GDM_KEY_GTK_MODULES_LIST);	
     GdmGroup = gnome_config_get_string (GDM_KEY_GROUP);
     GdmHalt = gnome_config_get_string (GDM_KEY_HALT);
     GdmKillInitClients = gnome_config_get_bool (GDM_KEY_KILLIC);
     GdmLogDir= gnome_config_get_string (GDM_KEY_LOGDIR);
     GdmPidFile = gnome_config_get_string (GDM_KEY_PIDFILE);
     GdmSessionDir = gnome_config_get_string (GDM_KEY_SESSDIR);
+    GdmPostLogin = gnome_config_get_string (GDM_KEY_POSTLOGIN);
     GdmPreSession = gnome_config_get_string (GDM_KEY_PRESESS);
     GdmPostSession = gnome_config_get_string (GDM_KEY_POSTSESS);
     GdmFailsafeXServer = gnome_config_get_string (GDM_KEY_FAILSAFE_XSERVER);
@@ -253,6 +268,7 @@ gdm_config_parse (void)
     GdmConfigurator = gnome_config_get_string (GDM_KEY_CONFIGURATOR);
     GdmConfigAvailable = gnome_config_get_bool (GDM_KEY_CONFIG_AVAILABLE);
     GdmSystemMenu = gnome_config_get_bool (GDM_KEY_SYSMENU);
+    GdmChooserButton = gnome_config_get_bool (GDM_KEY_CHOOSER_BUTTON);
     GdmBrowser = gnome_config_get_bool (GDM_KEY_BROWSER);
     GdmGlobalFaceDir = gnome_config_get_string (GDM_KEY_FACEDIR);
     GdmXineramaScreen = gnome_config_get_int (GDM_KEY_XINERAMASCREEN);
@@ -261,9 +277,21 @@ gdm_config_parse (void)
     GdmRootPath = gnome_config_get_string (GDM_KEY_ROOTPATH);
     GdmServAuthDir = gnome_config_get_string (GDM_KEY_SERVAUTH);
     GdmSessDir = gnome_config_get_string (GDM_KEY_SESSDIR);
+    GdmXsession = gnome_config_get_string (GDM_KEY_BASEXSESSION);
+    if (ve_string_empty (GdmXsession)) {
+	    gdm_info (_("%s: BaseXsession empty, using %s/gdm/Xsession"),
+			"gdm_config_parse",
+			EXPANDED_SYSCONFDIR);
+	    g_free (GdmXsession);
+	    GdmXsession = g_strconcat (EXPANDED_SYSCONFDIR, "/gdm/Xsession",
+				       NULL);
+    }
     GdmSuspend = gnome_config_get_string (GDM_KEY_SUSPEND);
     GdmLocaleFile = gnome_config_get_string (GDM_KEY_LOCFILE);
+#if 0
+/* FIXME: Maybe just whack this */
     GdmGnomeDefaultSession = gnome_config_get_string (GDM_KEY_GNOMEDEFAULTSESSION);
+#endif
     GdmUser = gnome_config_get_string (GDM_KEY_USER);
     GdmUserAuthDir = gnome_config_get_string (GDM_KEY_UAUTHDIR);
     GdmUserAuthFile = gnome_config_get_string (GDM_KEY_UAUTHFILE);
@@ -315,6 +343,7 @@ gdm_config_parse (void)
 
     GdmFirstVT = gnome_config_get_int (GDM_KEY_FIRSTVT);    
     GdmVTAllocation = gnome_config_get_bool (GDM_KEY_VTALLOCATION);    
+    GdmDisallowTCP = gnome_config_get_bool (GDM_KEY_DISALLOWTCP);    
 
     GdmDebug = gnome_config_get_bool (GDM_KEY_DEBUG);
 
@@ -356,7 +385,7 @@ gdm_config_parse (void)
     }
 
     if (GdmTimedLoginDelay < 5) {
-	    gdm_info (_("%s: TimedLoginDelay less then 5, so I will just use 5."), "gdm_config_parse");
+	    gdm_info (_("%s: TimedLoginDelay less than 5, so I will just use 5."), "gdm_config_parse");
 	    GdmTimedLoginDelay = 5;
     }
 
@@ -407,6 +436,8 @@ gdm_config_parse (void)
 			    (GDM_KEY_SERVER_CHOOSABLE);
 		    svr->handled = gnome_config_get_bool
 			    (GDM_KEY_SERVER_HANDLED);
+		    svr->chooser = gnome_config_get_bool
+			    (GDM_KEY_SERVER_CHOOSER);
 
 		    if (ve_string_empty (svr->command)) {
 			    gdm_error (_("%s: Empty server command, "
@@ -831,7 +862,7 @@ gdm_final_cleanup (void)
 		 * because X is stupid and full of races and will hang my
 		 * keyboard if I don't */
 		if (li != list)
-			gdm_sleep_no_signal (2);
+			sleep (2);
 		gdm_display_unmanage (d);
 	}
 	g_slist_free (list);
@@ -872,8 +903,10 @@ deal_with_x_crashes (GdmDisplay *d)
 	    /* Yay we have a failsafe */
 	    if ( ! ve_string_empty (bin) &&
 		access (bin, X_OK) == 0) {
-		    gdm_info (_("deal_with_x_crashes: Trying failsafe X "
-				"server %s"), GdmFailsafeXServer);
+		    gdm_info (_("%s: Trying failsafe X "
+				"server %s"), 
+			      "deal_with_x_crashes",
+			      GdmFailsafeXServer);
 		    g_free (bin);
 		    g_free (d->command);
 		    d->command = g_strdup (GdmFailsafeXServer);
@@ -888,8 +921,9 @@ deal_with_x_crashes (GdmDisplay *d)
 	access (GdmXKeepsCrashing, X_OK|R_OK) == 0) {
 	    pid_t pid;
 
-	    gdm_info (_("deal_with_x_crashes: Running the "
-			"XKeepsCrashing script"));
+	    gdm_info (_("%s: Running the "
+			"XKeepsCrashing script"),
+		      "deal_with_x_crashes");
 
 	    pid = gdm_fork_extra ();
 
@@ -919,6 +953,8 @@ deal_with_x_crashes (GdmDisplay *d)
 		    gnome_setenv ("XLOG", xlog, TRUE);
 		    gnome_setenv ("BINDIR", EXPANDED_BINDIR, TRUE);
 		    gnome_setenv ("SBINDIR", EXPANDED_SBINDIR, TRUE);
+		    gnome_setenv ("LIBEXECDIR", EXPANDED_LIBEXECDIR, TRUE);
+		    gnome_setenv ("SYSCONFDIR", EXPANDED_SYSCONFDIR, TRUE);
 
 		    /* To enable gettext stuff in the script */
 		    gnome_setenv ("TEXTDOMAIN", GETTEXT_PACKAGE, TRUE);
@@ -956,7 +992,7 @@ deal_with_x_crashes (GdmDisplay *d)
     /* if we have "open" we can talk to the user, not as user
      * friendly as the above script, but getting there */
     if ( ! just_abort &&
-	access (EXPANDED_SBINDIR "/gdmopen", X_OK) == 0) {
+	access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) == 0) {
 	    /* Shit if we knew what the program was to tell the user,
 	     * the above script would have been defined and we'd run
 	     * it for them */
@@ -984,8 +1020,6 @@ suspend_machine (void)
 	if (GdmSuspendReal != NULL &&
 	    fork () == 0) {
 		char **argv;
-		if (GdmXdmcp)
-			gdm_xdmcp_close ();
 		/* In the child setup empty mask and set all signals to
 		 * default values */
 		gdm_unset_signals ();
@@ -1002,6 +1036,30 @@ suspend_machine (void)
 	}
 }
 
+#ifdef __linux__
+static void
+change_to_first_and_clear (gboolean reboot)
+{
+	gdm_change_vt (1);
+	close (1);
+	close (2);
+	open ("/dev/tty1", O_WRONLY);
+	open ("/dev/tty1", O_WRONLY);
+	/* yet again lazy */
+	if (access ("/usr/bin/clear", X_OK) == 0)
+		system ("/usr/bin/clear");
+	else if (access ("/bin/clear", X_OK) == 0)
+		system ("/bin/clear");
+	printf ("\n\n\n");
+	if (reboot)
+		printf (_("System is rebooting, please wait ..."));
+	else
+		printf (_("System is shutting down, please wait ..."));
+	printf ("\n\n\n");
+}
+#endif /* __linux__ */
+
+
 static gboolean 
 gdm_cleanup_children (void)
 {
@@ -1078,19 +1136,33 @@ gdm_cleanup_children (void)
 
     if ( ! GdmSystemMenu &&
 	(status == DISPLAY_REBOOT ||
+	 status == DISPLAY_SUSPEND ||
 	 status == DISPLAY_HALT)) {
-	    gdm_info (_("gdm_child_action: Reboot or Halt request when there is no system menu from display %s"), d->name);
+	    gdm_info (_("Reboot or Halt request when there is no system menu from display %s"), d->name);
 	    status = DISPLAY_REMANAGE;
     }
 
     if ( ! d->console &&
 	(status == DISPLAY_RESTARTGDM ||
 	 status == DISPLAY_REBOOT ||
+	 status == DISPLAY_SUSPEND ||
 	 status == DISPLAY_HALT)) {
-	    gdm_info (_("gdm_child_action: Restart, Reboot or Halt request from a non-local display %s"), d->name);
+	    gdm_info (_("Restart, Reboot or Halt request from a non-local display %s"), d->name);
 	    status = DISPLAY_REMANAGE;
     }
 
+    if (status == DISPLAY_RUN_CHOOSER) {
+	    /* use the chooser on the next run (but only if allowed) */
+	    if (GdmSystemMenu &&
+		GdmChooserButton)
+		    d->use_chooser = TRUE;
+	    status = DISPLAY_REMANAGE;
+	    /* go around the display loop detection, these are short
+	     * sessions, so this decreases the chances of the loop
+	     * detection being hit */
+	    d->last_loop_start_time = 0;
+    }
+
     if (status == DISPLAY_CHOSEN) {
 	    /* forget about this indirect id, since this
 	     * display will be dead very soon, and we don't want it
@@ -1123,7 +1195,8 @@ start_autopsy:
     switch (status) {
 	
     case DISPLAY_ABORT:		/* Bury this display for good */
-	gdm_info (_("gdm_child_action: Aborting display %s"), d->name);
+	gdm_info (_("%s: Aborting display %s"),
+		  "gdm_child_action", d->name);
 
 	if (gdm_restart_mode)
 		gdm_safe_restart ();
@@ -1135,37 +1208,49 @@ start_autopsy:
 	break;
 	
     case DISPLAY_REBOOT:	/* Reboot machine */
-	gdm_info (_("gdm_child_action: Master rebooting..."));
+	gdm_info (_("Master rebooting..."));
 
 	gdm_final_cleanup ();
 	chdir ("/");
 
+#ifdef __linux__
+	change_to_first_and_clear (TRUE /* reboot */);
+#endif /* __linux */
+
 	argv = ve_split (GdmRebootReal);
 	execv (argv[0], argv);
 
-	gdm_error (_("gdm_child_action: Reboot failed: %s"), strerror (errno));
+	gdm_error (_("%s: Reboot failed: %s"), 
+		   "gdm_child_action", strerror (errno));
 
 	status = DISPLAY_REMANAGE;
 	goto start_autopsy;
 	break;
 	
     case DISPLAY_HALT:		/* Halt machine */
-	gdm_info (_("gdm_child_action: Master halting..."));
+	gdm_info (_("Master halting..."));
 
 	gdm_final_cleanup ();
 	chdir ("/");
 
+#ifdef __linux__
+	change_to_first_and_clear (FALSE /* reboot */);
+#endif /* __linux */
+
 	argv = ve_split (GdmHaltReal);
 	execv (argv[0], argv);
 
-	gdm_error (_("gdm_child_action: Halt failed: %s"), strerror (errno));
+	gdm_error (_("%s: Halt failed: %s"),
+		   "gdm_child_action", strerror (errno));
 
 	status = DISPLAY_REMANAGE;
 	goto start_autopsy;
 	break;
 
     case DISPLAY_SUSPEND:	/* Suspend machine */
-	gdm_info (_("gdm_child_action: Master suspending..."));
+	/* XXX: this is ugly, why should there be a suspend like this,
+	 * see GDM_SOP_SUSPEND_MACHINE */
+	gdm_info (_("Master suspending..."));
 	suspend_machine ();
 
 	status = DISPLAY_REMANAGE;
@@ -1192,7 +1277,7 @@ start_autopsy:
 	if (d->type == TYPE_LOCAL) {
 		time_t now = time (NULL);
 		d->x_faileds ++;
-		/* this is a much faster failing, don't even allow the 8
+		/* this is a much faster failing, don't even allow a few
 		 * seconds, just flash but for at most 30 seconds */
 		if (now - d->last_x_failed > 30) {
 			/* reset */
@@ -1218,7 +1303,7 @@ start_autopsy:
 		}
 		/* go around the display loop detection, we're doing
 		 * our own here */
-		d->last_start_time = 0;
+		d->last_loop_start_time = 0;
 	}
 	/* fall through */
 
@@ -1245,8 +1330,20 @@ start_autopsy:
 			 * start them now */
 			gdm_start_first_unborn_local (3 /* delay */);
 		}
+	} else if (d->type == TYPE_FLEXI || d->type == TYPE_FLEXI_XNEST) {
+		/* if this was a chooser session and we have chosen a host,
+		   then we don't want to unmanage, we want to manage and
+		   choose that host */
+		if (d->chosen_hostname != NULL) {
+			if ( ! gdm_display_manage (d)) {
+				gdm_display_unmanage (d);
+			}
+		} else {
+			/* else, this is a one time thing */
+			gdm_display_unmanage (d);
+		}
 	/* Remote displays will send a request to be managed */
-	} else /* TYPE_XDMCP, TYPE_FLEXI, TYPE_FLEXI_XNEST */ {
+	} else /* TYPE_XDMCP */ {
 		gdm_display_unmanage (d);
 	}
 	
@@ -1357,13 +1454,11 @@ create_connections (void)
 
 	unixconn = gdm_connection_open_unix (GDM_SUP_SOCKET, 0666);
 
-	if (unixconn != NULL) {
+	if (unixconn != NULL)
 		gdm_connection_set_handler (unixconn,
 					    gdm_handle_user_message,
 					    NULL /* data */,
 					    NULL /* destroy_notify */);
- 		gdm_connection_set_nonblock (unixconn, TRUE);
-	}
 }
 
 struct poptOption options [] = {
@@ -1420,6 +1515,24 @@ ensure_desc_012 (void)
 	}
 }
 
+static void
+gdm_get_our_runlevel (void)
+{
+#ifdef __linux__
+	/* on linux we get our current runlevel, for use later
+	 * to detect a shutdown going on, and not mess up. */
+	if (access ("/sbin/runlevel", X_OK) == 0) {
+		char ign;
+		int rnl;
+		FILE *fp = fopen ("/sbin/runlevel", "r");
+		if (fscanf (fp, "%c %d", &ign, &rnl) == 2) {
+			gdm_normal_runlevel = rnl;
+		}
+		fclose (fp);
+	}
+#endif /* __linux__ */
+}
+
 int 
 main (int argc, char *argv[])
 {
@@ -1438,6 +1551,8 @@ main (int argc, char *argv[])
     store_argv (argc, argv);
     gdm_saveenv ();
 
+    gdm_get_our_runlevel ();
+
     bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
     textdomain (GETTEXT_PACKAGE);
 
@@ -1648,7 +1763,7 @@ write_x_servers (GdmDisplay *d)
 		char **argv;
 		char *command;
 		argv = gdm_server_resolve_command_line
-			(d, FALSE /* resolve_handled */,
+			(d, FALSE /* resolve_flags */,
 			 NULL /* vtarg */);
 		command = g_strjoinv (" ", argv);
 		g_strfreev (argv);
@@ -1660,13 +1775,21 @@ write_x_servers (GdmDisplay *d)
 }
 
 static void
-send_slave_ack (GdmDisplay *d)
+send_slave_ack (GdmDisplay *d, const char *resp)
 {
 	if (d->master_notify_fd >= 0) {
-		char not[2];
-		not[0] = GDM_SLAVE_NOTIFY_ACK;
-		not[1] = '\n';
-		write (d->master_notify_fd, not, 2);
+		if (resp == NULL) {
+			char not[2];
+			not[0] = GDM_SLAVE_NOTIFY_ACK;
+			not[1] = '\n';
+			write (d->master_notify_fd, not, 2);
+		} else {
+			char *not = g_strdup_printf ("%c%s\n",
+						     GDM_SLAVE_NOTIFY_ACK,
+						     resp);
+			write (d->master_notify_fd, not, strlen (not));
+			g_free (not);
+		}
 	}
 	gdm_sigchld_block_push ();
 	if (d->slavepid > 1) {
@@ -1717,6 +1840,32 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 	if (strncmp (msg, GDM_SOP_CHOSEN " ",
 		     strlen (GDM_SOP_CHOSEN " ")) == 0) {
 		gdm_choose_data (msg);
+	} else if (strncmp (msg, GDM_SOP_CHOSEN_LOCAL " ",
+		            strlen (GDM_SOP_CHOSEN_LOCAL " ")) == 0) {
+		GdmDisplay *d;
+		long slave_pid;
+		char *p;
+
+		if (sscanf (msg, GDM_SOP_CHOSEN_LOCAL " %ld", &slave_pid)
+		    != 1)
+			return;
+		p = strchr (msg, ' ');
+		if (p != NULL)
+			p = strchr (p+1, ' ');
+		if (p == NULL)
+			return;
+		p++;
+
+		/* Find out who this slave belongs to */
+		d = gdm_display_lookup (slave_pid);
+
+		if (d != NULL) {
+			g_free (d->chosen_hostname);
+			d->chosen_hostname = g_strdup (p);
+			gdm_debug ("Got CHOSEN_LOCAL == %s", p);
+			/* send ack */
+			send_slave_ack (d, NULL);
+		}
 	} else if (strncmp (msg, GDM_SOP_XPID " ",
 		            strlen (GDM_SOP_XPID " ")) == 0) {
 		GdmDisplay *d;
@@ -1733,7 +1882,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->servpid = pid;
 			gdm_debug ("Got XPID == %ld", (long)pid);
 			/* send ack */
-			send_slave_ack (d);
+			send_slave_ack (d, NULL);
 		}
 	} else if (strncmp (msg, GDM_SOP_SESSPID " ",
 		            strlen (GDM_SOP_SESSPID " ")) == 0) {
@@ -1751,7 +1900,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->sesspid = pid;
 			gdm_debug ("Got SESSPID == %ld", (long)pid);
 			/* send ack */
-			send_slave_ack (d);
+			send_slave_ack (d, NULL);
 		}
 	} else if (strncmp (msg, GDM_SOP_GREETPID " ",
 		            strlen (GDM_SOP_GREETPID " ")) == 0) {
@@ -1769,7 +1918,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->greetpid = pid;
 			gdm_debug ("Got GREETPID == %ld", (long)pid);
 			/* send ack */
-			send_slave_ack (d);
+			send_slave_ack (d, NULL);
 		}
 	} else if (strncmp (msg, GDM_SOP_CHOOSERPID " ",
 		            strlen (GDM_SOP_CHOOSERPID " ")) == 0) {
@@ -1787,7 +1936,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->chooserpid = pid;
 			gdm_debug ("Got CHOOSERPID == %ld", (long)pid);
 			/* send ack */
-			send_slave_ack (d);
+			send_slave_ack (d, NULL);
 		}
 	} else if (strncmp (msg, GDM_SOP_LOGGED_IN " ",
 		            strlen (GDM_SOP_LOGGED_IN " ")) == 0) {
@@ -1813,7 +1962,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 				gdm_safe_restart ();
 
 			/* send ack */
-			send_slave_ack (d);
+			send_slave_ack (d, NULL);
 		}
 	} else if (strncmp (msg, GDM_SOP_DISP_NUM " ",
 		            strlen (GDM_SOP_DISP_NUM " ")) == 0) {
@@ -1834,7 +1983,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->dispnum = disp_num;
 			gdm_debug ("Got DISP_NUM == %d", disp_num);
 			/* send ack */
-			send_slave_ack (d);
+			send_slave_ack (d, NULL);
 		}
 	} else if (strncmp (msg, GDM_SOP_VT_NUM " ",
 		            strlen (GDM_SOP_VT_NUM " ")) == 0) {
@@ -1853,7 +2002,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->vt = vt_num;
 			gdm_debug ("Got VT_NUM == %d", vt_num);
 			/* send ack */
-			send_slave_ack (d);
+			send_slave_ack (d, NULL);
 		}
 	} else if (strncmp (msg, GDM_SOP_LOGIN " ",
 		            strlen (GDM_SOP_LOGIN " ")) == 0) {
@@ -1880,7 +2029,53 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->login = g_strdup (p);
 			gdm_debug ("Got LOGIN == %s", p);
 			/* send ack */
-			send_slave_ack (d);
+			send_slave_ack (d, NULL);
+		}
+	} else if (strncmp (msg, GDM_SOP_QUERYLOGIN " ",
+		            strlen (GDM_SOP_QUERYLOGIN " ")) == 0) {
+		GdmDisplay *d;
+		long slave_pid;
+		char *p;
+
+		if (sscanf (msg, GDM_SOP_QUERYLOGIN " %ld",
+			    &slave_pid) != 1)
+			return;
+		p = strchr (msg, ' ');
+		if (p != NULL)
+			p = strchr (p+1, ' ');
+		if (p == NULL)
+			return;
+
+		p++;
+
+		/* Find out who this slave belongs to */
+		d = gdm_display_lookup (slave_pid);
+
+		if (d != NULL) {
+			GString *resp = NULL;
+			GSList *li;
+			gdm_debug ("Got QUERYLOGIN %s", p);
+			for (li = displays; li != NULL; li = li->next) {
+				GdmDisplay *di = li->data;
+				if (di->login != NULL &&
+				    strcmp (di->login, p) == 0) {
+					if (resp == NULL) {
+						resp = g_string_new (di->name);
+						g_string_append_printf (resp, ",%d", di->vt);
+					} else {
+						g_string_append_printf (resp, ",%s,%d",
+									di->name, di->vt);
+					}
+				}
+			}
+
+			/* send ack */
+			if (resp != NULL) {
+				send_slave_ack (d, resp->str);
+				g_string_free (resp, TRUE);
+			} else {
+				send_slave_ack (d, NULL);
+			}
 		}
 	} else if (strncmp (msg, GDM_SOP_COOKIE " ",
 		            strlen (GDM_SOP_COOKIE " ")) == 0) {
@@ -1907,7 +2102,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			d->cookie = g_strdup (p);
 			gdm_debug ("Got COOKIE == <secret>");
 			/* send ack */
-			send_slave_ack (d);
+			send_slave_ack (d, NULL);
 		}
 	} else if (strncmp (msg, GDM_SOP_FLEXI_ERR " ",
 		            strlen (GDM_SOP_FLEXI_ERR " ")) == 0) {
@@ -1944,7 +2139,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			
 			gdm_debug ("Got FLEXI_ERR == %d", err);
 			/* send ack */
-			send_slave_ack (d);
+			send_slave_ack (d, NULL);
 		}
 	} else if (strncmp (msg, GDM_SOP_FLEXI_OK " ",
 		            strlen (GDM_SOP_FLEXI_OK " ")) == 0) {
@@ -1974,7 +2169,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			
 			gdm_debug ("Got FLEXI_OK");
 			/* send ack */
-			send_slave_ack (d);
+			send_slave_ack (d, NULL);
 		}
 	} else if (strcmp (msg, GDM_SOP_SOFT_RESTART) == 0) {
 		gdm_restart_mode = TRUE;
@@ -2043,8 +2238,15 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			write_x_servers (d);
 
 			/* send ack */
-			send_slave_ack (d);
+			send_slave_ack (d, NULL);
 		}
+	} else if (strcmp (msg, GDM_SOP_SUSPEND_MACHINE) == 0) {
+		gdm_info (_("Master suspending..."));
+		if (GdmSuspendReal != NULL &&
+		    GdmSystemMenu) {
+			suspend_machine ();
+		}
+
 	}
 }
 
@@ -2246,6 +2448,7 @@ check_cookie (const char *file, const char *disp, const char *cookie)
 static void
 handle_flexi_server (GdmConnection *conn, int type, const char *server,
 		     gboolean handled,
+		     gboolean chooser,
 		     const char *xnest_disp, uid_t xnest_uid,
 		     const char *xnest_auth_file,
 		     const char *xnest_cookie)
@@ -2318,7 +2521,12 @@ handle_flexi_server (GdmConnection *conn, int type, const char *server,
 				      "ERROR 2 Startup errors\n");
 		return;
 	}
+
+	/* It is kind of ugly that we don't use
+	   the standard resolution for this, but
+	   oh well, this makes other things simpler */
 	display->handled = handled;
+	display->use_chooser = chooser;
 
 	if (type == TYPE_FLEXI_XNEST) {
 		GdmDisplay *parent;
@@ -2490,6 +2698,15 @@ update_config (const char *key)
 
 		notify_displays_int (GDM_NOTIFY_CONFIG_AVAILABLE, val);
 
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_CHOOSER_BUTTON)) {
+		gboolean val = gnome_config_get_bool (GDM_KEY_CHOOSER_BUTTON);
+		if (ve_bool_equal (val, GdmChooserButton))
+			goto update_config_ok;
+		GdmChooserButton = val;
+
+		notify_displays_int (GDM_NOTIFY_CHOOSER_BUTTON, val);
+
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_RETRYDELAY)) {
 		int val = gnome_config_get_int (GDM_KEY_RETRYDELAY);
@@ -2499,6 +2716,15 @@ update_config (const char *key)
 
 		notify_displays_int (GDM_NOTIFY_RETRYDELAY, val);
 
+		goto update_config_ok;
+	} else if (is_key (key, GDM_KEY_DISALLOWTCP)) {
+		gboolean val = gnome_config_get_bool (GDM_KEY_DISALLOWTCP);
+		if (ve_bool_equal (val, GdmDisallowTCP))
+			goto update_config_ok;
+		GdmDisallowTCP = val;
+
+		notify_displays_int (GDM_NOTIFY_DISALLOWTCP, val);
+
 		goto update_config_ok;
 	} else if (is_key (key, GDM_KEY_GREETER)) {
 		char *val = gnome_config_get_string (GDM_KEY_GREETER);
@@ -2613,12 +2839,6 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 {
 	gdm_debug ("Handling user message: '%s'", msg);
 
-	if (gdm_connection_get_message_count (conn) > 20) {
-		gdm_connection_write (conn, "ERROR 200 Too many messages\n");
-		gdm_connection_close (conn);
-		return;
-	}
-
 	if (strncmp (msg, GDM_SUP_AUTH_LOCAL " ",
 		     strlen (GDM_SUP_AUTH_LOCAL " ")) == 0) {
 		GSList *li;
@@ -2664,6 +2884,7 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 		}
 		handle_flexi_server (conn, TYPE_FLEXI, GdmStandardXServer,
 				     TRUE /* handled */,
+				     FALSE /* chooser */,
 				     NULL, 0, NULL, NULL);
 	} else if (strncmp (msg, GDM_SUP_FLEXI_XSERVER " ",
 		            strlen (GDM_SUP_FLEXI_XSERVER " ")) == 0) {
@@ -2705,7 +2926,12 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 		}
 		g_free (name);
 
-		handle_flexi_server (conn, TYPE_FLEXI, command, svr->handled,
+		handle_flexi_server (conn, TYPE_FLEXI, command,
+				     /* It is kind of ugly that we don't use
+					the standard resolution for this, but
+					oh well, this makes other things simpler */
+				     svr->handled,
+				     svr->chooser,
 				     NULL, 0, NULL, NULL);
 	} else if (strncmp (msg, GDM_SUP_FLEXI_XNEST " ",
 		            strlen (GDM_SUP_FLEXI_XNEST " ")) == 0) {
@@ -2735,6 +2961,7 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 		
 		handle_flexi_server (conn, TYPE_FLEXI_XNEST, GdmXnest,
 				     TRUE /* handled */,
+				     FALSE /* chooser */,
 				     dispname, uid, xauthfile, cookie);
 
 		g_free (dispname);
@@ -2763,6 +2990,20 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 			}
 		}
 		gdm_connection_write (conn, "\n");
+	} else if (strcmp (msg, GDM_SUP_ALL_SERVERS) == 0) {
+		GSList *li;
+		const char *sep = " ";
+		gdm_connection_write (conn, "OK");
+		for (li = displays; li != NULL; li = li->next) {
+			GdmDisplay *disp = li->data;
+			gdm_connection_printf (conn,
+					       "%s%s,%s",
+					       sep,
+					       ve_sure_string (disp->name),
+					       ve_sure_string (disp->login));
+			sep = ";";
+		}
+		gdm_connection_write (conn, "\n");
 	} else if (strcmp (msg, GDM_SUP_GREETERPIDS) == 0) {
 		GSList *li;
 		const char *sep = " ";
