@@ -37,7 +37,6 @@
 #ifdef HAVE_SYS_SOCKIO_H
 #include <sys/sockio.h>
 #endif
-#include <sys/resource.h>
 
 #include <vicious.h>
 
@@ -47,7 +46,6 @@
 
 /* Configuration option variables */
 extern gchar *GdmPidFile;
-extern gchar *GdmLocalNoPasswordUsers;
 extern gboolean GdmDebug;
 extern GSList *displays;
 extern int gdm_xdmcpfd;
@@ -320,7 +318,6 @@ gdm_get_free_display (int start, uid_t server_uid)
 		GSList *li;
 		struct stat s;
 		char buf[256];
-		FILE *fp;
 
 		for (li = displays; li != NULL; li = li->next) {
 			GdmDisplay *dsp = li->data;
@@ -346,32 +343,6 @@ gdm_get_free_display (int start, uid_t server_uid)
 		}
 		close (sock);
 
-		/* if lock file exists and the process exists */
-		g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", i);
-		if (stat (buf, &s) == 0 &&
-		    ! S_ISREG (s.st_mode)) {
-			/* Eeeek! not a regular file?  Perhaps someone
-			   is trying to play tricks on us */
-			continue;
-		}
-		fp = fopen (buf, "r");
-		if (fp != NULL) {
-			char buf2[100];
-			if (fgets (buf2, sizeof (buf2), fp) != NULL) {
-				gulong pid;
-				if (sscanf (buf2, "%lu", &pid) == 1 &&
-				    kill (pid, 0) == 0) {
-					fclose (fp);
-					continue;
-				}
-
-			}
-			fclose (fp);
-
-			/* whack the file, it's a stale lock file */
-			unlink (buf);
-		}
-
 		/* if starting as root, we'll be able to overwrite any
 		 * stale sockets or lock files, but a user may not be
 		 * able to */
@@ -403,7 +374,7 @@ gdm_text_message_dialog (const char *msg)
 	char *dialog; /* do we have dialog?*/
 	char *msg_quoted;
 
-	if (access (EXPANDED_SBINDIR "/gdmopen", X_OK) != 0)
+	if (access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) != 0)
 		return FALSE;
 
 	msg_quoted = g_shell_quote (msg);
@@ -416,7 +387,7 @@ gdm_text_message_dialog (const char *msg)
 	if (dialog != NULL) {
 		char *argv[6];
 		
-		argv[0] = EXPANDED_SBINDIR "/gdmopen";
+		argv[0] = EXPANDED_LIBEXECDIR "/gdmopen";
 		argv[1] = "-l";
 		argv[2] = "/bin/sh";
 		argv[3] = "-c";
@@ -429,16 +400,14 @@ gdm_text_message_dialog (const char *msg)
 				   TRUE /* de_setuid */) < 0) {
 			g_free (dialog);
 			g_free (msg_quoted);
-			g_free (argv[4]);
 			return FALSE;
 		}
 
 		g_free (dialog);
-		g_free (argv[4]);
 	} else {
 		char *argv[6];
 
-		argv[0] = EXPANDED_SBINDIR "/gdmopen";
+		argv[0] = EXPANDED_LIBEXECDIR "/gdmopen";
 		argv[1] = "-l";
 		argv[2] = "/bin/sh";
 		argv[3] = "-c";
@@ -450,11 +419,11 @@ gdm_text_message_dialog (const char *msg)
 
 		if (gdm_exec_wait (argv, TRUE /* no display */,
 				   TRUE /* de_setuid */) < 0) {
-			g_free (argv[4]);
+			g_free (argv[3]);
 			g_free (msg_quoted);
 			return FALSE;
 		}
-		g_free (argv[4]);
+		g_free (argv[3]);
 	}
 	g_free (msg_quoted);
 	return TRUE;
@@ -465,8 +434,8 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 {
 	char *dialog; /* do we have dialog?*/
 	char *msg_quoted;
-
-	if (access (EXPANDED_SBINDIR "/gdmopen", X_OK) != 0)
+	
+	if (access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) != 0)
 		return FALSE;
 
 	if (ret != NULL)
@@ -483,7 +452,7 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 		char *argv[6];
 		int retint;
 
-		argv[0] = EXPANDED_SBINDIR "/gdmopen";
+		argv[0] = EXPANDED_LIBEXECDIR "/gdmopen";
 		argv[1] = "-l";
 		argv[2] = "/bin/sh";
 		argv[3] = "-c";
@@ -496,7 +465,6 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 		retint = gdm_exec_wait (argv, TRUE /* no display */,
 					TRUE /* de_setuid */);
 		if (retint < 0) {
-			g_free (argv[4]);
 			g_free (dialog);
 			g_free (msg_quoted);
 			return FALSE;
@@ -505,7 +473,6 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 		if (ret != NULL)
 			*ret = (retint == 0) ? TRUE : FALSE;
 
-		g_free (argv[4]);
 		g_free (dialog);
 		g_free (msg_quoted);
 
@@ -525,7 +492,7 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 
 		close (tempfd);
 
-		argv[0] = EXPANDED_SBINDIR "/gdmopen";
+		argv[0] = EXPANDED_LIBEXECDIR "/gdmopen";
 		argv[1] = "-l";
 		argv[2] = "/bin/sh";
 		argv[3] = "-c";
@@ -541,11 +508,11 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 
 		if (gdm_exec_wait (argv, TRUE /* no display */,
 				   TRUE /* de_setuid */) < 0) {
-			g_free (argv[4]);
+			g_free (argv[3]);
 			g_free (msg_quoted);
 			return FALSE;
 		}
-		g_free (argv[4]);
+		g_free (argv[3]);
 
 		if (ret != NULL) {
 			fp = fopen (tempname, "r");
@@ -752,42 +719,27 @@ void
 gdm_ensure_sanity (void)
 {
 	mode_t old_umask;
-	uid_t old_euid;
-	gid_t old_egid;
-
-	old_euid = geteuid ();
-	old_egid = getegid ();
-
-	seteuid (0);
-	setegid (0);
 
 	/* The /tmp/.ICE-unix check, note that we do
 	 * ignore errors, since it's not deadly to run
 	 * if we can't perform this task :) */
 	old_umask = umask (0);
 
-        if (mkdir ("/tmp/.ICE-unix", 01777) != 0) {
-		/* if we can't create it, perhaps it
-		   already exists, in which case ensure the
-		   correct permissions */
+        if (mkdir ("/tmp/.ICE-unix", 0777) == 0) {
+		/* Make sure it is root */
+		if (chown ("/tmp/.ICE-unix", 0, 0) == 0)
+			chmod ("/tmp/.ICE-unix", 01777);
+        } else {
 		struct stat s;
-		int r;
-		IGNORE_EINTR (r = lstat ("/tmp/.ICE-unix", &s));
-		if (r == 0 && S_ISDIR (s.st_mode)) {
+		if (lstat ("/tmp/.ICE-unix", &s) == 0 &&
+		    S_ISDIR (s.st_mode)) {
 			/* Make sure it is root and sticky */
-			IGNORE_EINTR (chown ("/tmp/.ICE-unix", 0, 0));
-			IGNORE_EINTR (chmod ("/tmp/.ICE-unix", 01777));
-		} else {
-			/* There is a file/link/whatever called .ICE-unix? whack and try mkdir */
-			unlink ("/tmp/.ICE-unix");
-			mkdir ("/tmp/.ICE-unix", 01777);
+			if (chown ("/tmp/.ICE-unix", 0, 0) == 0)
+				chmod ("/tmp/.ICE-unix", 01777);
 		}
 	}
 
 	umask (old_umask);
-
-	seteuid (old_euid);
-	setegid (old_egid);
 }
 
 const GList *
@@ -883,7 +835,7 @@ gdm_peek_local_address_list (void)
 	if (gethostname (hostbuf, BUFSIZ-1) != 0) {
 		gdm_debug ("%s: Could not get server hostname: %s!",
 			   "gdm_peek_local_address_list",
-			   g_strerror (errno));
+			   strerror (errno));
 		addr = g_new0 (struct in_addr, 1);
 		addr->s_addr = INADDR_LOOPBACK;
 		return g_list_append (NULL, addr);
@@ -1105,28 +1057,6 @@ gdm_close_all_descriptors (int from, int except)
 	}
 }
 
-gboolean
-gdm_is_a_no_password_user (const char *user)
-{
-	char **vector = NULL;
-	int i;
-
-	if (ve_string_empty (GdmLocalNoPasswordUsers) ||
-	    ve_string_empty (user) ||
-	    strcmp (user, "root") == 0)
-		return FALSE;
-
-	vector = g_strsplit (GdmLocalNoPasswordUsers, ",", -1);
-	for (i = 0; vector[i] != NULL; i++) {
-		if (strcmp (vector[i], user) == 0) {
-			g_strfreev (vector);
-			return TRUE;
-		}
-	}
-	g_strfreev (vector);
-	return FALSE;
-}
-
 int
 gdm_open_dev_null (mode_t mode)
 {
@@ -1150,7 +1080,6 @@ gdm_unset_signals (void)
 	signal (SIGUSR1, SIG_DFL);
 	signal (SIGUSR2, SIG_DFL);
 	signal (SIGCHLD, SIG_DFL);
-	signal (SIGINT, SIG_DFL);
 	signal (SIGTERM, SIG_DFL);
 	signal (SIGPIPE, SIG_DFL);
 	signal (SIGALRM, SIG_DFL);
@@ -1215,73 +1144,6 @@ gdm_locale_from_utf8 (const char *text)
 	return out;
 }
 
-void
-gdm_sleep_no_signal (int secs)
-{
-	time_t endtime = time (NULL)+secs;
-
-	while (secs > 0) {
-		struct timeval tv;
-		tv.tv_sec = secs;
-		tv.tv_usec = 0;
-		select (0, NULL, NULL, NULL, &tv);
-		/* don't want to use sleep since we're using alarm
-		   for pinging */
-		secs = endtime - time (NULL);
-	}
-}
-
-void
-gdm_reset_limits (void)
-{
-	struct rlimit unlim = { RLIM_INFINITY, RLIM_INFINITY };
-
-	/* Note: I don't really know which ones are really very standard
-	   and which ones are not, so I just test for them all one by one */
-
-#ifdef RLIMIT_CPU
-	setrlimit (RLIMIT_CPU, &unlim);
-#endif
-#ifdef RLIMIT_DATA
-	setrlimit (RLIMIT_DATA, &unlim);
-#endif
-#ifdef RLIMIT_FSIZE
-	setrlimit (RLIMIT_FSIZE, &unlim);
-#endif
-#ifdef RLIMIT_LOCKS
-	setrlimit (RLIMIT_LOCKS, &unlim);
-#endif
-#ifdef RLIMIT_MEMLOCK
-	setrlimit (RLIMIT_MEMLOCK, &unlim);
-#endif
-#ifdef RLIMIT_NOFILE
-	setrlimit (RLIMIT_NOFILE, &unlim);
-#endif
-#ifdef RLIMIT_OFILE
-	setrlimit (RLIMIT_OFILE, &unlim);
-#endif
-#ifdef RLIMIT_NPROC
-	setrlimit (RLIMIT_NPROC, &unlim);
-#endif
-#ifdef RLIMIT_RSS
-	setrlimit (RLIMIT_RSS, &unlim);
-#endif
-#ifdef RLIMIT_STACK
-	setrlimit (RLIMIT_STACK, &unlim);
-#endif
-#ifdef RLIMIT_CORE
-	setrlimit (RLIMIT_CORE, &unlim);
-#endif
-#ifdef RLIMIT_AS
-	setrlimit (RLIMIT_AS, &unlim);
-#endif
-#ifdef RLIMIT_VMEM
-	setrlimit (RLIMIT_VMEM, &unlim);
-#endif
-#ifdef RLIMIT_PTHREAD
-	setrlimit (RLIMIT_PTHREAD, &unlim);
-#endif
-}
 
 
 /* EOF */
