@@ -75,8 +75,6 @@ gboolean gdm_test_opt (const char *cmd, const char *help, const char *option);
 
 void gdm_close_all_descriptors (int from, int except);
 
-gboolean gdm_is_a_no_password_user (const char *user);
-
 int gdm_open_dev_null (mode_t mode);
 
 void gdm_unset_signals (void);
@@ -88,9 +86,6 @@ void gdm_saveenv (void);
 /* leaks */
 void gdm_restoreenv (void);
 
-void gdm_sleep_no_signal (int secs);
-void gdm_reset_limits (void);
-
 #endif /* GDM_MISC_H */
 
 /* EOF */
