@@ -64,6 +64,7 @@ extern gchar *GdmServAuthDir;
 extern gchar *GdmLogDir;
 extern gchar *GdmStandardXServer;
 extern gboolean GdmXdmcp;
+extern gboolean GdmDisallowTCP;
 extern gint high_display_num;
 extern pid_t extra_process;
 extern int extra_status;
@@ -78,8 +79,6 @@ static int server_signal_pipe[2];
 void
 gdm_server_wipe_cookies (GdmDisplay *disp)
 {
-	char buf[256];
-
 	if ( ! ve_string_empty (disp->authfile))
 		unlink (disp->authfile);
 	g_free (disp->authfile);
@@ -88,27 +87,6 @@ gdm_server_wipe_cookies (GdmDisplay *disp)
 		unlink (disp->authfile_gdm);
 	g_free (disp->authfile_gdm);
 	disp->authfile_gdm = NULL;
-
-	/* X seems to be sometimes broken with its lock files and
-	   doesn't seem to remove them always, and if you manage
-	   to get into the weird state where the old pid now
-	   corresponds to some new pid, X will just die with
-	   a stupid error. */
-
-	/* Yes there could be a race here if another X server starts
-	   at this exact instant.  Oh well such is life.  Very unlikely
-	   to happen though as we should really be the only ones
-	   trying to start X servers, and we aren't starting an
-	   X server on this display yet. */
-
-	/* if lock file exists and it is our process, whack it! */
-	g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", disp->dispnum);
-	unlink (buf);
-
-	/* whack the unix socket as well */
-	g_snprintf (buf, sizeof (buf),
-		    "/tmp/.X11-unix/X%d", disp->dispnum);
-	unlink (buf);
 }
 
 /* ignore handlers */
@@ -169,7 +147,7 @@ gdm_server_reinit (GdmDisplay *disp)
 		 * get whacked ourselves after we open the connection and we'll think
 		 * it's an X screwup, which is really OK to happen and will just
 		 * restart the Xserver, it's just more nasty.  Oh how fun */
-		gdm_sleep_no_signal (1);
+		sleep (1);
 	} else if (pid == 0) {
 		Display *dsp;
 
@@ -199,7 +177,7 @@ gdm_server_reinit (GdmDisplay *disp)
 
 		if (dsp == NULL) {
 			/* HACK, see note above */
-			gdm_sleep_no_signal (1);
+			sleep (1);
 			_exit (0);
 		} else {
 			/* Wait for an xioerror */
@@ -260,7 +238,6 @@ gdm_server_stop (GdmDisplay *disp)
 		    ve_waitpid_no_signal (servpid, 0, 0);
 	    gdm_sigchld_block_pop ();
 
-
 	    gdm_debug ("gdm_server_stop: Server pid %d dead", (int)servpid);
     }
 
@@ -273,7 +250,7 @@ static gboolean
 busy_ask_user (GdmDisplay *disp)
 {
     /* if we have "open" we can talk to the user */
-    if (access (EXPANDED_SBINDIR "/gdmopen", X_OK) == 0) {
+    if (access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) == 0) {
 	    char *error = g_strdup_printf
 		    (_("There already appears to be an X server "
 		       "running on display %s.  Should I try another "
@@ -479,7 +456,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 
     if (pipe (server_signal_pipe) != 0) {
 	    gdm_error (_("%s: Error opening a pipe: %s"),
-		       "gdm_server_start", g_strerror (errno));
+		       "gdm_server_start", strerror (errno));
 	    return FALSE; 
     }
     server_signal_notified = FALSE;
@@ -491,7 +468,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 
     if (sigaction (SIGUSR1, &usr1, &old_usr1) < 0) {
 	    gdm_error (_("%s: Error setting up USR1 signal handler: %s"),
-		       "gdm_server_start", g_strerror (errno));
+		       "gdm_server_start", strerror (errno));
 	    close (server_signal_pipe[0]);
 	    close (server_signal_pipe[1]);
 	    return FALSE;
@@ -504,7 +481,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 
     if (sigaction (SIGCHLD, &chld, &old_chld) < 0) {
 	    gdm_error (_("%s: Error setting up CHLD signal handler: %s"),
-		       "gdm_server_start", g_strerror (errno));
+		       "gdm_server_start", strerror (errno));
 	    sigaction (SIGUSR1, &old_usr1, NULL);
 	    close (server_signal_pipe[0]);
 	    close (server_signal_pipe[1]);
@@ -518,7 +495,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 
     if (sigaction (SIGALRM, &alrm, &old_alrm) < 0) {
 	    gdm_error (_("%s: Error setting up ALRM signal handler: %s"),
-		       "gdm_server_start", g_strerror (errno));
+		       "gdm_server_start", strerror (errno));
 	    sigaction (SIGUSR1, &old_usr1, NULL);
 	    sigaction (SIGCHLD, &old_chld, NULL);
 	    close (server_signal_pipe[0]);
@@ -563,7 +540,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 		     * just wait a few seconds and hope things just work,
 		     * fortunately there is no such case yet and probably
 		     * never will, but just for code anality's sake */
-		    gdm_sleep_no_signal (5);
+		    sleep (5);
 		    /* In case we got a SIGCHLD */
 		    check_child_status ();
 	    } else if (d->server_uid != 0) {
@@ -586,7 +563,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 			 i++) {
 			    d->dsp = XOpenDisplay (d->name);
 			    if (d->dsp == NULL)
-				    gdm_sleep_no_signal (1);
+				    sleep (1);
 			    else
 				    d->servstat = SERVER_RUNNING;
 
@@ -600,13 +577,14 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 			    d->servstat = SERVER_TIMEOUT;
 		    }
 	    } else {
+		    fd_set rfds;
 
 		    gdm_debug ("gdm_server_start: Before mainloop waiting for server");
 
+		    FD_ZERO (&rfds);
+		    FD_SET (server_signal_pipe[0], &rfds);
+
 		    do {
-			    fd_set rfds;
-			    FD_ZERO (&rfds);
-			    FD_SET (server_signal_pipe[0], &rfds);
 			    select (server_signal_pipe[0]+1, &rfds, NULL, NULL, NULL);
 			    /* In case we got a SIGCHLD */
 			    check_child_status ();
@@ -795,7 +773,7 @@ rotate_logs (const char *dname)
 
 char **
 gdm_server_resolve_command_line (GdmDisplay *disp,
-				 gboolean resolve_handled,
+				 gboolean resolve_flags,
 				 const char *vtarg)
 {
 	char *bin;
@@ -803,6 +781,7 @@ gdm_server_resolve_command_line (GdmDisplay *disp,
 	int len;
 	int i;
 	gboolean gotvtarg = FALSE;
+	gboolean query_in_arglist = FALSE;
 
 	bin = ve_first_word (disp->command);
 	if (bin == NULL) {
@@ -829,9 +808,15 @@ gdm_server_resolve_command_line (GdmDisplay *disp,
 				g_strfreev (old_argv);
 			} 
 
-			if (resolve_handled)
+			if (resolve_flags) {
 				/* Setup the handled function */
 				disp->handled = svr->handled;
+				/* never make use_chooser FALSE,
+				   it may have been set temporarily for
+				   us by the master */
+				if (svr->chooser)
+					disp->use_chooser = TRUE;
+			}
 		}
 	} else {
 		argv = ve_split (disp->command);
@@ -846,20 +831,41 @@ gdm_server_resolve_command_line (GdmDisplay *disp,
 		    (arg[3] == '\0' ||
 		     (isdigit (arg[3]) && arg[4] == '\0')))
 			gotvtarg = TRUE;
+		if (strcmp (arg, "-query") == 0 ||
+		    strcmp (arg, "-indirect") == 0)
+			query_in_arglist = TRUE;
 	}
 
-	argv = g_renew (char *, argv, len + 5);
+	argv = g_renew (char *, argv, len + 10);
 	for (i = len - 1; i >= 1; i--) {
 		argv[i+1] = argv[i];
 	}
 	/* server number is the FIRST argument, before any others */
 	argv[1] = g_strdup (disp->name);
 	len++;
+
 	if (disp->authfile != NULL) {
 		argv[len++] = g_strdup ("-auth");
 		argv[len++] = g_strdup (disp->authfile);
 	}
 
+	if (resolve_flags && disp->chosen_hostname) {
+		/* this display is NOT handled */
+		disp->handled = FALSE;
+		/* never ever ever use chooser here */
+		disp->use_chooser = FALSE;
+		/* run just one session */
+		argv[len++] = g_strdup ("-terminate");
+		argv[len++] = g_strdup ("-query");
+		argv[len++] = g_strdup (disp->chosen_hostname);
+		query_in_arglist = TRUE;
+	}
+
+	if (resolve_flags && GdmDisallowTCP && ! query_in_arglist) {
+		argv[len++] = g_strdup ("-nolisten");
+		argv[len++] = g_strdup ("tcp");
+	}
+
 	if (vtarg != NULL &&
 	    ! gotvtarg) {
 		argv[len++] = g_strdup (vtarg);
@@ -912,7 +918,7 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 
     /* Figure out the server command */
     argv = gdm_server_resolve_command_line (d,
-					    TRUE /* resolve handled */,
+					    TRUE /* resolve flags */,
 					    vtarg);
 
     command = g_strjoinv (" ", argv);
@@ -958,7 +964,8 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 		dup2 (logfd, 1);
 		dup2 (logfd, 2);
         } else {
-		gdm_error (_("gdm_server_spawn: Could not open logfile for display %s!"), d->name);
+		gdm_error (_("%s: Could not open logfile for display %s!"),
+			   "gdm_server_spawn", d->name);
 	}
 
 	
@@ -968,15 +975,18 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 	sigemptyset (&ign_signal.sa_mask);
 
 	if (sigaction (SIGUSR1, &ign_signal, NULL) < 0) {
-		gdm_error (_("gdm_server_spawn: Error setting USR1 to SIG_IGN"));
+		gdm_error (_("%s: Error setting %s to %s"),
+			   "gdm_server_spawn", "USR1", "SIG_IGN");
 		_exit (SERVER_ABORT);
 	}
 	if (sigaction (SIGTTIN, &ign_signal, NULL) < 0) {
-		gdm_error (_("gdm_server_spawn: Error setting TTIN to SIG_IGN"));
+		gdm_error (_("%s: Error setting %s to %s"),
+			   "gdm_server_spawn", "TTIN", "SIG_IGN");
 		_exit (SERVER_ABORT);
 	}
 	if (sigaction (SIGTTOU, &ign_signal, NULL) < 0) {
-		gdm_error (_("gdm_server_spawn: Error setting TTOU to SIG_IGN"));
+		gdm_error (_("%s: Error setting %s to %s"),
+			   "gdm_server_spawn", "TTOU", "SIG_IGN");
 		_exit (SERVER_ABORT);
 	}
 
@@ -986,11 +996,13 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 	sigemptyset (&dfl_signal.sa_mask);
 
 	if (sigaction (SIGHUP, &dfl_signal, NULL) < 0) {
-		gdm_error (_("gdm_server_spawn: Error setting HUP to SIG_DFL"));
+		gdm_error (_("%s: Error setting %s to %s"),
+			   "gdm_server_spawn", "HUP", "SIG_DFL");
 		_exit (SERVER_ABORT);
 	}
 	if (sigaction (SIGTERM, &dfl_signal, NULL) < 0) {
-		gdm_error (_("gdm_server_spawn: Error setting TERM to SIG_DFL"));
+		gdm_error (_("%s: Error setting %s to %s"),
+			   "gdm_server_spawn", "TERM", "SIG_DFL");
 		_exit (SERVER_ABORT);
 	}
 
@@ -1082,14 +1094,16 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 
 	execv (argv[0], argv);
 	
-	gdm_error (_("gdm_server_spawn: Xserver not found: %s"), command);
+	gdm_error (_("%s: Xserver not found: %s"), 
+		   "gdm_server_spawn", command);
 	
 	_exit (SERVER_ABORT);
 	
     case -1:
 	g_strfreev (argv);
 	g_free (command);
-	gdm_error (_("gdm_server_spawn: Can't fork Xserver process!"));
+	gdm_error (_("%s: Can't fork Xserver process!"),
+		   "gdm_server_spawn");
 	d->servpid = 0;
 	d->servstat = SERVER_DEAD;
 
@@ -1098,7 +1112,8 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
     default:
 	g_strfreev (argv);
 	g_free (command);
-	gdm_debug ("gdm_server_spawn: Forked server on pid %d", (int)pid);
+	gdm_debug ("%s: Forked server on pid %d", 
+		   "gdm_server_spawn", (int)pid);
 	break;
     }
 }
@@ -1224,6 +1239,7 @@ gdm_server_alloc (gint id, const gchar *command)
 
     d->x_servers_order = -1;
 
+    d->last_loop_start_time = 0;
     d->last_start_time = 0;
     d->retry_count = 0;
     d->disabled = FALSE;
