@@ -64,8 +64,6 @@ get_free_display (void)
 	for (i = 20; i < 3000; i ++) {
 		struct stat s;
 		char buf[256];
-		FILE *fp;
-
 		sock = socket (AF_INET, SOCK_STREAM, 0);
 
 		serv_addr.sin_port = htons (6000 + i);
@@ -80,23 +78,6 @@ get_free_display (void)
 
 		close (sock);
 
-		/* if lock file exists and the process exists */
-		g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", i);
-		fp = fopen (buf, "r");
-		if (fp != NULL) {
-			char buf2[100];
-			if (fgets (buf2, sizeof (buf2), fp) != NULL) {
-				gulong pid;
-				if (sscanf (buf2, "%lu", &pid) == 1 &&
-				    kill (pid, 0) == 0) {
-					fclose (fp);
-					continue;
-				}
-
-			}
-			fclose (fp);
-		}
-
 		g_snprintf (buf, sizeof (buf), "/tmp/.X11-unix/X%d", i);
 		if (stat (buf, &s) == 0 &&
 		    s.st_uid != getuid ()) {
@@ -488,6 +469,7 @@ main (int argc, char *argv[])
 					    "Please ask your system "
 					    "administrator\n"
 					    "to install it."));
+		gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 		gtk_widget_show_all (d);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (d);
@@ -501,8 +483,9 @@ main (int argc, char *argv[])
 		FILE *fp = NULL;
 		long pid;
 		
-		if ( ! xdmcp_enabled ||
-		     ! honor_indirect) {
+		if (( ! xdmcp_enabled ||
+		      ! honor_indirect) &&
+		    ! do_direct) {
 			GtkWidget *d;
 			d = gtk_message_dialog_new
 				(NULL /* parent */,
@@ -514,6 +497,27 @@ main (int argc, char *argv[])
 				   "system administrator to enable "
 				   "it\nin the GDM configurator "
 				   "program."));
+			gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
+			gtk_widget_show_all (d);
+			gtk_dialog_run (GTK_DIALOG (d));
+			gtk_widget_destroy (d);
+			return 0;
+		}
+
+		if ( ! xdmcp_enabled &&
+		    do_direct) {
+			GtkWidget *d;
+			d = gtk_message_dialog_new
+				(NULL /* parent */,
+				 GTK_DIALOG_MODAL /* flags */,
+				 GTK_MESSAGE_WARNING,
+				 GTK_BUTTONS_OK,
+				 _("XDMCP is not enabled,\n"
+				   "please ask your "
+				   "system administrator to enable "
+				   "it\nin the GDM configurator "
+				   "program."));
+			gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 			gtk_widget_show_all (d);
 			gtk_dialog_run (GTK_DIALOG (d));
 			gtk_widget_destroy (d);
@@ -541,6 +545,7 @@ main (int argc, char *argv[])
 				 _("GDM is not running.\n"
 				   "Please ask your "
 				   "system administrator to start it."));
+			gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 			gtk_widget_show_all (d);
 			gtk_dialog_run (GTK_DIALOG (d));
 			gtk_widget_destroy (d);
@@ -557,6 +562,7 @@ main (int argc, char *argv[])
 					    GTK_BUTTONS_OK,
 					    _("Could not find a free "
 					      "display number"));
+		gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 		gtk_widget_show_all (d);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (d);
