@@ -134,6 +134,7 @@ choose_server (void)
 					      GTK_STOCK_CANCEL,
 					      GTK_RESPONSE_CANCEL,
 					      NULL);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	vbox = GTK_DIALOG (dialog)->vbox;
 
 	w = gtk_label_new (_("Choose the X server to start"));
@@ -251,7 +252,8 @@ main (int argc, char *argv[])
 				 GTK_BUTTONS_OK,
 				 _("Cannot communicate with gdm, perhaps "
 				   "you have an old version running."));
-			gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+			gtk_dialog_set_has_separator (GTK_DIALOG (dialog),
+						      FALSE);
 			gtk_widget_show_all (dialog);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
@@ -271,7 +273,8 @@ main (int argc, char *argv[])
 				   "authentication needed for this "
 				   "operation.  Perhaps your .Xauthority "
 				   "file is not set up correctly."));
-			gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+			gtk_dialog_set_has_separator (GTK_DIALOG (dialog),
+						      FALSE);
 			gtk_widget_show_all (dialog);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
@@ -297,7 +300,8 @@ main (int argc, char *argv[])
 				 _("You do not seem to be logged in on the "
 				   "console.  Starting a new login only "
 				   "works correctly on the console."));
-			gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+			gtk_dialog_set_has_separator (GTK_DIALOG (dialog),
+						      FALSE);
 			gtk_widget_show_all (dialog);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
