@@ -147,6 +147,7 @@ check_update_error:
 						  "login screens.  Not all "
 						  "updates may have taken "
 						  "effect."));
+		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
 		shown_error = TRUE;
@@ -375,7 +376,6 @@ timeout_remove_all (void)
 	timeout_widgets = NULL;
 
 	for (li = list; li != NULL; li = li->next) {
-		g_print ("Removing %p!\n", li->data);
 		timeout_remove (li->data);
 		li->data = NULL;
 	}
@@ -414,6 +414,7 @@ setup_user_combo (const char *name, const char *key)
 	GList *users = NULL;
 	struct passwd *pwent;
 	char *str;
+	int cnt;
 
 	str = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE), key);
 
@@ -426,12 +427,20 @@ setup_user_combo (const char *name, const char *key)
 	setpwent ();
 
 	pwent = getpwent();
+	cnt = 0;
 	
 	while (pwent != NULL) {
 		if (pwent->pw_uid >= GdmMinimalUID &&
 		    strcmp (ve_sure_string (str), pwent->pw_name) != 0) {
+			cnt ++;
 			users = g_list_append (users,
 					       g_strdup (pwent->pw_name));
+			/* FIXME: fix properly, see bug #111830 */
+			if (cnt >= 50) {
+				users = g_list_append (users,
+						       g_strdup (_("Too many users to list here...")));
+				break;
+			}
 		}
 	
 		pwent = getpwent();
@@ -654,8 +663,6 @@ greeter_entry_untranslate_timeout (GtkWidget *entry)
 	const char *text;
 	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
 
-	g_print ("Timeout %p!\n", entry);
-
 	text = gtk_entry_get_text (GTK_ENTRY (entry));
 
 	ve_config_delete_translations (config, key);
@@ -672,7 +679,6 @@ greeter_entry_untranslate_timeout (GtkWidget *entry)
 static void
 greeter_entry_untranslate_changed (GtkWidget *entry)
 {
-	g_print ("Changed %p!\n", entry);
 	run_timeout (entry, 500, greeter_entry_untranslate_timeout);
 }
 
@@ -837,7 +843,7 @@ xdmcp_toggled (GtkWidget *toggle, gpointer data)
 {
 	GtkWidget *frame = data;
 
-	gtk_widget_set_sensitive (GTK_BIN (frame)->child,
+	gtk_widget_set_sensitive (frame,
 				  GTK_TOGGLE_BUTTON (toggle)->active);
 }
 
@@ -845,17 +851,23 @@ static void
 setup_xdmcp_support (void)
 {
 	GtkWidget *xdmcp_toggle = glade_helper_get (xml, "enable_xdmcp", GTK_TYPE_TOGGLE_BUTTON);
-	GtkWidget *xdmcp_frame = glade_helper_get (xml, "xdmcp_frame", GTK_TYPE_FRAME);
+	GtkWidget *xdmcp_frame = glade_helper_get (xml, "xdmcp_frame", GTK_TYPE_WIDGET);
+	GtkWidget *xdmcp_vbox = glade_helper_get (xml, "xdmcp_vbox", GTK_TYPE_WIDGET);
+	GtkWidget *no_xdmcp_label = glade_helper_get (xml, "no_xdmcp_label", GTK_TYPE_WIDGET);
 
 #ifndef HAVE_LIBXDMCP
-	gtk_widget_show (glade_helper_get (xml, "no_xdmcp_label", GTK_TYPE_LABEL));
-	gtk_widget_set_sensitive (xdmcp_toggle, FALSE);
-	gtk_widget_set_sensitive (xdmcp_frame, FALSE);
-#else /* ! HAVE_LIBXDMCP */
-	gtk_widget_hide (glade_helper_get (xml, "no_xdmcp_label", GTK_TYPE_LABEL));
+	gtk_widget_show (no_xdmcp_label);
+	gtk_widget_hide (xdmcp_vbox);
+#else /* HAVE_LIBXDMCP */
+	gtk_widget_hide (no_xdmcp_label);
+	gtk_widget_show (xdmcp_vbox);
 #endif /* HAVE_LIBXDMCP */
 
-	gtk_widget_set_sensitive (GTK_BIN (xdmcp_frame)->child, 
+	/* Why doesn't glade have this? */
+	gtk_label_set_use_markup (GTK_LABEL (GTK_BIN (xdmcp_toggle)->child),
+				  TRUE);
+
+	gtk_widget_set_sensitive (xdmcp_frame, 
 				  GTK_TOGGLE_BUTTON (xdmcp_toggle)->active);
 
 	g_signal_connect (G_OBJECT (xdmcp_toggle), "toggled",
@@ -972,13 +984,28 @@ selection_changed (GtkTreeSelection *selection, gpointer data)
 					       GTK_TYPE_IMAGE);
         GtkWidget *no_preview = glade_helper_get (xml, "gg_theme_no_preview",
 						  GTK_TYPE_WIDGET);
+	GtkWidget *del_button = glade_helper_get (xml, "gg_delete_theme",
+						  GTK_TYPE_BUTTON);
         char *screenshot;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	GValue value = {0, };
 
-	if ( ! gtk_tree_selection_get_selected (selection, &model, &iter))
+	if ( ! gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		gtk_widget_set_sensitive (del_button, FALSE);
 		return;
+	}
+
+	gtk_widget_set_sensitive (del_button, TRUE);
+
+	gtk_tree_model_get_value (model, &iter,
+				  THEME_COLUMN_SELECTED,
+				  &value);
+	/* Do not allow deleting of selected theme */
+	if (g_value_get_boolean (&value)) {
+		gtk_widget_set_sensitive (del_button, FALSE);
+	}
+	g_value_unset (&value);
 
 	gtk_tree_model_get_value (model, &iter,
 				  THEME_COLUMN_AUTHOR,
@@ -1486,6 +1513,7 @@ install_ok (GtkWidget *button, gpointer data)
 						GTK_MESSAGE_ERROR,
 						GTK_BUTTONS_OK,
 						_("No file selected"));
+		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
 		g_free (cwd);
@@ -1514,6 +1542,7 @@ install_ok (GtkWidget *button, gpointer data)
 						_("Not a theme archive\n"
 						  "Details: %s"),
 						error);
+		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
 		g_free (filename);
@@ -1537,6 +1566,7 @@ install_ok (GtkWidget *button, gpointer data)
 			   "installed, install again anyway?"),
 			 fname);
 		g_free (fname);
+		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		if (gtk_dialog_run (GTK_DIALOG (dlg)) != GTK_RESPONSE_YES) {
 			gtk_widget_destroy (dlg);
 			g_free (filename);
@@ -1553,20 +1583,23 @@ install_ok (GtkWidget *button, gpointer data)
 	if (chdir (theme_dir) == 0) {
 		if (system (untar_cmd) == 0) {
 			char *cmd;
+			char *quoted = g_shell_quote (dir);
 			success = TRUE;
 
 			/* HACK! */
-			cmd = g_strdup_printf ("/bin/chown -R root.root %s", dir);
+			cmd = g_strdup_printf ("/bin/chown -R root.root %s", quoted);
 			system (cmd);
 			g_free (cmd);
 
-			cmd = g_strdup_printf ("/bin/chmod -R a+r %s", dir);
+			cmd = g_strdup_printf ("/bin/chmod -R a+r %s", quoted);
 			system (cmd);
 			g_free (cmd);
 
-			cmd = g_strdup_printf ("/bin/chmod a+x %s", dir);
+			cmd = g_strdup_printf ("/bin/chmod a+x %s", quoted);
 			system (cmd);
 			g_free (cmd);
+
+			g_free (quoted);
 		}
 		chdir (cwd);
 	}
@@ -1580,6 +1613,7 @@ install_ok (GtkWidget *button, gpointer data)
 						GTK_BUTTONS_OK,
 						_("Some error occured when "
 						  "installing the theme"));
+		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
 	}
@@ -1638,6 +1672,109 @@ install_new_theme (GtkWidget *button, gpointer data)
 	gtk_widget_show (fs);
 }
 
+static void
+delete_theme (GtkWidget *button, gpointer data)
+{
+	GtkListStore *store = data;
+	GtkWidget *theme_list;
+	GtkWidget *setup_dialog;
+	GtkWidget *del_button;
+	GtkTreeSelection *selection;
+        char *dir, *name;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GValue value = {0, };
+	GtkWidget *dlg;
+
+	setup_dialog = glade_helper_get (xml, "setup_dialog", GTK_TYPE_WINDOW);
+	theme_list = glade_helper_get (xml, "gg_theme_list",
+				       GTK_TYPE_TREE_VIEW);
+	del_button = glade_helper_get (xml, "gg_delete_theme",
+				       GTK_TYPE_BUTTON);
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (theme_list));
+
+	if ( ! gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		/* should never get here since the button shuld not be
+		 * enabled */
+		return;
+	}
+
+	gtk_tree_model_get_value (model, &iter,
+				  THEME_COLUMN_SELECTED,
+				  &value);
+	/* Do not allow deleting of selected theme */
+	if (g_value_get_boolean (&value)) {
+		/* should never get here since the button shuld not be
+		 * enabled */
+		g_value_unset (&value);
+		return;
+	}
+	g_value_unset (&value);
+
+	gtk_tree_model_get_value (model, &iter,
+				  THEME_COLUMN_NAME,
+				  &value);
+	name = g_strdup (g_value_get_string (&value));
+	g_value_unset (&value);
+
+	gtk_tree_model_get_value (model, &iter,
+				  THEME_COLUMN_DIR,
+				  &value);
+	dir = g_strdup (g_value_get_string (&value));
+	g_value_unset (&value);
+
+	dlg = gtk_message_dialog_new
+		(GTK_WINDOW (setup_dialog),
+		 GTK_DIALOG_MODAL | 
+		 GTK_DIALOG_DESTROY_WITH_PARENT,
+		 GTK_MESSAGE_QUESTION,
+		 GTK_BUTTONS_YES_NO,
+		 _("Do you really wish to remove theme '%s' from the system?"),
+		 name);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+	if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_YES) {
+		char *theme_dir = get_theme_dir ();
+		char *cwd = g_get_current_dir ();
+		if (chdir (theme_dir) == 0 &&
+		    /* this is a security sanity check, since we're doing rm -fR */
+		    strchr (dir, '/') == NULL) {
+			/* HACK! */
+			DIR *dp;
+			GtkTreeIter *select_iter = NULL;
+			char *quoted = g_shell_quote (dir);
+			char *cmd = g_strdup_printf ("/bin/rm -fR %s", quoted);
+			system (cmd);
+			g_free (cmd);
+			g_free (quoted);
+
+			/* Update the list */
+			gtk_list_store_clear (store);
+
+			dp = opendir (theme_dir);
+
+			if (dp != NULL) {
+				select_iter = read_themes (store, theme_dir, dp, 
+							   selected_theme);
+				closedir (dp);
+			}
+
+			if (select_iter != NULL) {
+				gtk_tree_selection_select_iter (selection, select_iter);
+				g_free (select_iter);
+			}
+
+		}
+		chdir (cwd);
+		g_free (cwd);
+		g_free (theme_dir);
+	}
+	gtk_widget_destroy (dlg);
+
+	g_free (name);
+	g_free (dir);
+}
+
 static void
 setup_graphical_themes (void)
 {
@@ -1651,6 +1788,8 @@ setup_graphical_themes (void)
 						  GTK_TYPE_TREE_VIEW);
 	GtkWidget *button = glade_helper_get (xml, "gg_install_new_theme",
 					      GTK_TYPE_BUTTON);
+	GtkWidget *del_button = glade_helper_get (xml, "gg_delete_theme",
+						  GTK_TYPE_BUTTON);
 
 	char *theme_dir = get_theme_dir ();
 
@@ -1670,6 +1809,9 @@ setup_graphical_themes (void)
 
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (install_new_theme), store);
+	g_signal_connect (del_button, "clicked",
+			  G_CALLBACK (delete_theme), store);
+	gtk_widget_set_sensitive (del_button, FALSE);
 
 	dir = opendir (theme_dir);
 
@@ -1771,6 +1913,7 @@ dialog_response (GtkWidget *dlg, int response, gpointer data)
 			   "are listed here.  You may want to edit %s "
 			   "if you cannot find what you are looking for."),
 			 GDM_CONFIG_FILE);
+		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		g_signal_connect (G_OBJECT (dlg), "destroy",
 				  G_CALLBACK (gtk_widget_destroyed),
 				  &dlg);
@@ -1781,10 +1924,18 @@ dialog_response (GtkWidget *dlg, int response, gpointer data)
 	}
 }
 
+static void
+add_to_size_group (GtkSizeGroup *sg, const char *wid)
+{
+	GtkWidget *w = glade_helper_get (xml, wid, GTK_TYPE_WIDGET);
+	gtk_size_group_add_widget (sg, w);
+}
+
 static void
 setup_gui (void)
 {
 	GtkWidget *dialog;
+	GtkSizeGroup *sg;
 
 	xml = glade_helper_load ("gdmsetup.glade",
 				 "setup_dialog",
@@ -1801,6 +1952,34 @@ setup_gui (void)
 	setup_greeter_backselect ();
 	setup_graphical_themes ();
 
+       	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	add_to_size_group (sg, "local_greeter");
+	add_to_size_group (sg, "remote_greeter");
+	add_to_size_group (sg, "welcome");
+	add_to_size_group (sg, "remote_welcome");
+	add_to_size_group (sg, "autologin_combo");
+	add_to_size_group (sg, "timedlogin_combo");
+	add_to_size_group (sg, "timedlogin_seconds");
+
+       	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	add_to_size_group (sg, "local_greeter_label");
+	add_to_size_group (sg, "remote_greeter_label");
+	add_to_size_group (sg, "welcome_label");
+	add_to_size_group (sg, "remote_welcome_label");
+	add_to_size_group (sg, "autologin_label");
+	add_to_size_group (sg, "timed_login_label");
+	add_to_size_group (sg, "timedlogin_seconds_label");
+
+       	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	add_to_size_group (sg, "greeter_table");
+	add_to_size_group (sg, "autologin_table");
+	add_to_size_group (sg, "timed_login_table");
+
+       	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	add_to_size_group (sg, "sg_use_24_clock");
+	add_to_size_group (sg, "autologin");
+	add_to_size_group (sg, "timedlogin");
+
 	setup_user_combo ("autologin_combo",
 			  GDM_KEY_AUTOMATICLOGIN);
 	setup_user_combo ("timedlogin_combo",
@@ -1828,6 +2007,12 @@ setup_gui (void)
 	setup_notify_toggle ("config_available",
 			     GDM_KEY_CONFIG_AVAILABLE,
 			     GDM_KEY_CONFIG_AVAILABLE /* notify_key */);
+	setup_notify_toggle ("chooser_button",
+			     GDM_KEY_CHOOSER_BUTTON,
+			     GDM_KEY_CHOOSER_BUTTON /* notify_key */);
+	setup_notify_toggle ("disallow_tcp",
+			     GDM_KEY_DISALLOWTCP,
+			     GDM_KEY_DISALLOWTCP /* notify_key */);
 
 	setup_notify_toggle ("enable_xdmcp",
 			     GDM_KEY_XDMCP,
@@ -1890,8 +2075,10 @@ setup_gui (void)
 	setup_greeter_color ("sg_backcolor",
 			     GDM_KEY_BACKGROUNDCOLOR);
 
-	setup_greeter_untranslate_entry ("sg_welcome",
+	setup_greeter_untranslate_entry ("welcome",
 					 GDM_KEY_WELCOME);
+	setup_greeter_untranslate_entry ("remote_welcome",
+					 GDM_KEY_REMOTEWELCOME);
 }
 
 static gboolean
@@ -2068,6 +2255,7 @@ main (int argc, char *argv[])
 						GTK_MESSAGE_ERROR,
 						GTK_BUTTONS_OK,
 						_("You must be the superuser (root) to configure GDM.\n"));
+		gtk_dialog_set_has_separator (GTK_DIALOG (fatal_error), FALSE);
 		if (RUNNING_UNDER_GDM)
 			setup_cursor (GDK_LEFT_PTR);
 		gtk_dialog_run (GTK_DIALOG (fatal_error));
