@@ -7,10 +7,10 @@ extern gboolean DOING_GDM_DEVELOPMENT;
 extern GtkWidget *canvas;
 extern GtkWidget *window;
 
+extern gboolean greeter_probably_login_prompt;
+
 void     greeter_abort   (const gchar *format, ...);
 gboolean greeter_query   (const gchar *msg);
 void     greeter_message (const gchar *msg);
 
-void     greeter_setup_cursor (GdkCursorType type);
-
 #endif
