@@ -108,12 +108,6 @@ greeter_language_get_language (const char *old_language)
 {
   gchar *retval = NULL;
 
-#if 0
-  if (greeter_current_user == NULL)
-    greeter_abort ("greeter_language_get_language: curuser==NULL. Mail <mkp@mkp.net> with " \
-		    "information on your PAM and user database setup");
-#endif
-  
   /* Don't save language unless told otherwise */
   savelang = FALSE;
 
@@ -202,9 +196,10 @@ greeter_action_language (GreeterItemInfo *info,
 					    GTK_STOCK_OK,
 					    GTK_RESPONSE_OK,
 					    NULL);
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+
       gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 				       GTK_RESPONSE_OK);
-      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
       g_object_add_weak_pointer (G_OBJECT (dialog), (gpointer *) &dialog);
       s = g_strdup_printf ("<span size=\"x-large\" weight=\"bold\">%s</span>",
 			   _("Select a language for your session to use:"));
