@@ -4,8 +4,9 @@
 #include <libgnome/libgnome.h>
 #include <gdk/gdkkeysyms.h>
 #include "greeter_item_pam.h"
+#include "greeter_item_ulist.h"
 #include "greeter_parser.h"
-#include "greeter.h"
+#include "greeter_configuration.h"
 #include "gdm.h"
 #include "gdmwm.h"
 #include "vicious.h"
@@ -14,73 +15,25 @@
 static gboolean messages_to_give = FALSE;
 static gboolean replace_msg = TRUE;
 static guint err_box_clear_handler = 0;
-static gboolean entry_is_login = FALSE;
-
-extern gboolean DOING_GDM_DEVELOPMENT;
-extern gboolean GdmSystemMenu;
 
 gchar *greeter_current_user = NULL;
 
-gboolean require_quarter = FALSE;
-
 void
 greeter_item_pam_set_user (const char *user)
 {
   g_free (greeter_current_user);
   greeter_current_user = g_strdup (user);
-}
-
-static gboolean
-evil (GtkEntry *entry, const char *user)
-{
-	/* do not translate */
-	if (strcmp (user, "Gimme Random Cursor") == 0) {
-		greeter_setup_cursor (((rand () >> 3) % (GDK_LAST_CURSOR/2)) * 2);
-		gtk_entry_set_text (GTK_ENTRY (entry), "");
-		return TRUE;
-		/* do not translate */
-	} else if (strcmp (user, "Require Quater") == 0 ||
-		   strcmp (user, "Require Quarter") == 0) {
-		/* btw, note that I misspelled quarter before and
-		 * thus this checks for Quater as well as Quarter to
-		 * keep compatibility which is obviously important for
-		 * something like this */
-		require_quarter = TRUE;
-		gtk_entry_set_text (GTK_ENTRY (entry), "");
-		return TRUE;
-	}
-
-	return FALSE;
+  greeter_item_ulist_set_user (user);
 }
 
 static void
 user_pw_activate (GtkEntry *entry, GreeterItemInfo *info)
 {
   char *tmp;
-  const char *str;
   GreeterItemInfo *error_info;
-
-  str = gtk_entry_get_text (GTK_ENTRY (entry));
-  if (entry_is_login &&
-      /* evilness */
-      evil (entry, str))
-    {
-      gtk_entry_set_text (GTK_ENTRY (entry), "");
-      return;
-    }
   
   gtk_widget_set_sensitive (GTK_WIDGET (entry), FALSE);
 
-  /* Save login. I'm making the assumption that login is always
-   * the first thing entered. This might not be true for all PAM
-   * setups. Needs thinking! 
-   */
-  
-  if (entry_is_login) {
-    g_free (greeter_current_user);
-    greeter_current_user = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
-  }
-  
   /* clear the err_box */
   if (err_box_clear_handler > 0)
     {
@@ -93,8 +46,6 @@ user_pw_activate (GtkEntry *entry, GreeterItemInfo *info)
 		  "text", "",
 		  NULL);
   
-  entry_is_login = FALSE;
-
   tmp = ve_locale_from_utf8 (gtk_entry_get_text (GTK_ENTRY (entry)));
   printf ("%c%s\n", STX, tmp);
   fflush (stdout);
@@ -124,7 +75,8 @@ greeter_item_pam_setup (void)
 
   entry_info = greeter_lookup_id ("user-pw-entry");
   if (entry_info && entry_info->item &&
-      GNOME_IS_CANVAS_WIDGET (entry_info->item))
+      GNOME_IS_CANVAS_WIDGET (entry_info->item) &&
+      GTK_IS_ENTRY (GNOME_CANVAS_WIDGET (entry_info->item)->widget))
     {
       GtkWidget *entry;
       entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
@@ -154,15 +106,12 @@ greeter_item_pam_setup (void)
 void
 greeter_item_pam_prompt (const char *message,
 			 int         entry_len,
-			 gboolean    entry_visible,
-			 gboolean    is_login)
+			 gboolean    entry_visible)
 {
   GreeterItemInfo *conversation_info;
   GreeterItemInfo *entry_info;
   GtkWidget *entry;
 
-  entry_is_login = is_login;
-  
   conversation_info = greeter_lookup_id ("pam-prompt");
   entry_info = greeter_lookup_id ("user-pw-entry");
 
@@ -363,6 +312,7 @@ greeter_item_pam_leftover_messages (void)
 					GTK_BUTTONS_OK,
 					"%s",
 					oldtext);
+	  gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 	  gtk_window_set_modal (GTK_WINDOW (dlg), TRUE);
 	  gdm_wm_center_window (GTK_WINDOW (dlg));
 
