@@ -267,6 +267,8 @@ greeter_session_init (void)
   vbox = gtk_vbox_new (FALSE, 6);
   /* we will pack this later depending on size */
 
+  sessnames = g_hash_table_new (g_str_hash, g_str_equal);
+
   if (GdmShowLastSession)
     {
       current_session = g_strdup (LAST_SESSION);
@@ -333,7 +335,7 @@ greeter_session_init (void)
 		    continue;
 	    }
 
-	    radio = gtk_radio_button_new_with_mnemonic (session_group, _(dent->d_name));
+	    radio = gtk_radio_button_new_with_mnemonic (session_group, name);
 	    g_object_set_data_full (G_OBJECT (radio),
 				    SESSION_NAME,
 				    g_strdup (dent->d_name),
