@@ -373,7 +373,7 @@ gboolean
 ve_locale_exists (const char *loc)
 {
 	gboolean ret;
-	char *old = g_strdup (setlocale (LC_ALL, NULL));
+	char *old = g_strdup (setlocale (LC_MESSAGES, NULL));
 	if (setlocale (LC_MESSAGES, loc) != NULL)
 		ret = TRUE;
 	else
